@@ -259,6 +259,9 @@ static char const rcsid[]= "$RCSfile: SQLAllocHandle.c,v $ $Revision: 1.13 $";
  */
 
 extern int pooling_enabled;
+extern int pool_max_size;
+extern int pool_wait_timeout;
+
 
 /*
  * this is used so that it can be called without falling
@@ -273,9 +276,12 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
     switch( handle_type )
     {
       case SQL_HANDLE_ENV:
+      case SQL_HANDLE_SENV:
         {
             DMHENV environment;
             char pooling_string[ 128 ];
+            char pool_max_size_string [ 128 ];
+            char pool_wait_timeout_string [ 128 ];
 
             if ( !output_handle ) 
             {
@@ -307,6 +313,19 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
                 pooling_enabled = 0;
             }
 
+            if ( pooling_enabled )
+            {
+                SQLGetPrivateProfileString( "ODBC", "PoolMaxSize", "0",
+                    pool_max_size_string, sizeof( pool_max_size_string ),
+                    "ODBCINST.INI" );
+                pool_max_size = atoi( pool_max_size_string );
+
+                SQLGetPrivateProfileString( "ODBC", "PoolWaitTimeout", "30",
+                    pool_wait_timeout_string, sizeof( pool_wait_timeout_string ),
+                    "ODBCINST.INI" );
+                pool_wait_timeout = atoi( pool_wait_timeout_string );
+            }
+
             if ( !( environment = __alloc_env()))
             {
                 *output_handle = SQL_NULL_HENV;
@@ -320,6 +339,7 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
 
             environment -> state = STATE_E1;
             environment -> requested_version = requested_version;
+            environment -> version_set = !!requested_version;
         	environment -> sql_driver_count = -1;
 
             /*
@@ -390,7 +410,7 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
                         ERROR_HY009, NULL, 
                         SQL_OV_ODBC3 );
 
-                return function_return( SQL_HANDLE_ENV, environment, SQL_ERROR );
+                return function_return_nodrv( SQL_HANDLE_ENV, environment, SQL_ERROR );
             }
 
             /*
@@ -411,7 +431,7 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
 
                 *output_handle = SQL_NULL_HDBC;
 
-                return function_return( SQL_HANDLE_ENV, environment, SQL_ERROR );
+                return function_return_nodrv( SQL_HANDLE_ENV, environment, SQL_ERROR );
             }
 
             connection = __alloc_dbc();
@@ -429,7 +449,7 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
 
                 *output_handle = SQL_NULL_HDBC;
 
-                return function_return( SQL_HANDLE_ENV, environment, SQL_ERROR );
+                return function_return_nodrv( SQL_HANDLE_ENV, environment, SQL_ERROR );
             }
 
             /*
@@ -447,9 +467,7 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
             connection -> cursors = SQL_CUR_DEFAULT;
             connection -> login_timeout = SQL_LOGIN_TIMEOUT_DEFAULT;
             connection -> login_timeout_set = 0;
-            connection -> auto_commit = 0;
-            connection -> auto_commit_set = 0;
-            connection -> auto_commit = 0;
+            connection -> auto_commit = SQL_AUTOCOMMIT_ON;
             connection -> auto_commit_set = 0;
             connection -> async_enable = 0;
             connection -> async_enable_set = 0;
@@ -483,6 +501,26 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
 
             *output_handle = (SQLHANDLE) connection;
 
+#ifndef ENABLE_DRIVER_ICONV
+
+            /*
+             * initialize unicode
+             */
+
+            if ( !unicode_setup( connection ))
+            {
+                char txt[ 256 ];
+
+                sprintf( txt, "Can't initiate unicode conversion" );
+
+                dm_log_write( __FILE__,
+                        __LINE__,
+                        LOG_INFO,
+                        LOG_INFO,
+                        txt );
+            }
+#endif
+
             if ( log_info.log_flag )
             {
                 sprintf( environment -> msg, 
@@ -555,7 +593,7 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
                         ERROR_HY009, NULL, 
                         connection -> environment -> requested_version  );
 
-                return function_return( SQL_HANDLE_DBC, connection , SQL_ERROR );
+                return function_return_nodrv( SQL_HANDLE_DBC, connection , SQL_ERROR );
             }
 
             if ( connection -> state == STATE_C1 ||
@@ -574,7 +612,7 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
 
                 *output_handle = SQL_NULL_HSTMT;
 
-                return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+                return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
             }
 
             statement = __alloc_stmt();
@@ -592,7 +630,7 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
 
                 *output_handle = SQL_NULL_HSTMT;
 
-                return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+                return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
             }
 
             /*
@@ -638,7 +676,7 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
 
                     *output_handle = SQL_NULL_HSTMT;
 
-                    return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+                    return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
                 }
             }
             else
@@ -680,7 +718,7 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
 
                     *output_handle = SQL_NULL_HSTMT;
                 
-                    return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+                    return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
                 }
             }
 
@@ -743,7 +781,7 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
 
                             __release_stmt( statement );
 
-                            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+                            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR, DEFER_R1 );
                         }
                         statement -> implicit_ard = statement -> ard;
                         statement -> ard -> implicit = 1;
@@ -787,7 +825,7 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
 
                             *output_handle = SQL_NULL_HSTMT;
 
-                            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+                            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR, DEFER_R1 );
                         }
                         statement -> implicit_apd = statement -> apd;
                         statement -> apd -> implicit = 1;
@@ -831,7 +869,7 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
 
                             *output_handle = SQL_NULL_HSTMT;
 
-                            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+                            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR, DEFER_R1 );
                         }
                         statement -> implicit_ird = statement -> ird;
                         statement -> ird -> implicit = 1;
@@ -875,7 +913,7 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
 
                             *output_handle = SQL_NULL_HSTMT;
 
-                            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+                            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR, DEFER_R1 );
                         }
                         statement -> implicit_ipd = statement -> ipd;
                         statement -> ipd -> implicit = 1;
@@ -922,7 +960,7 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
 
                             __release_stmt( statement );
 
-                            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+                            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR, DEFER_R1 );
                         }
                         statement -> implicit_ard = statement -> ard;
                         statement -> ard -> implicit = 1;
@@ -966,7 +1004,7 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
 
                             *output_handle = SQL_NULL_HSTMT;
 
-                            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+                            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR, DEFER_R1 );
                         }
                         statement -> implicit_apd = statement -> apd;
                         statement -> apd -> implicit = 1;
@@ -1010,7 +1048,7 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
 
                             *output_handle = SQL_NULL_HSTMT;
 
-                            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+                            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR, DEFER_R1 );
                         }
                         statement -> implicit_ird = statement -> ird;
                         statement -> ird -> implicit = 1;
@@ -1054,7 +1092,7 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
 
                             *output_handle = SQL_NULL_HSTMT;
 
-                            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+                            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR, DEFER_R1 );
                         }
                         statement -> implicit_ipd = statement -> ipd;
                         statement -> ipd -> implicit = 1;
@@ -1092,7 +1130,7 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
                                UODBC_STATS_TYPE_HSTMT, (void *)1);
 #endif
 
-            return function_return( SQL_HANDLE_DBC, connection, ret );
+            return function_return( SQL_HANDLE_DBC, connection, ret, DEFER_R1 );
         }
         break;
 
@@ -1146,7 +1184,7 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
                     ERROR_HY009, NULL, 
                     connection -> environment -> requested_version  );
 
-            return function_return( SQL_HANDLE_DBC, connection , SQL_ERROR );
+            return function_return_nodrv( SQL_HANDLE_DBC, connection , SQL_ERROR );
         }
 
         if ( connection -> state == STATE_C1 ||
@@ -1165,7 +1203,7 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
 
             *output_handle = SQL_NULL_HDESC;
 
-            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+            return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
         }
 
         descriptor = __alloc_desc();
@@ -1183,7 +1221,7 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
 
             *output_handle = SQL_NULL_HDESC;
 
-            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+            return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
         }
 
         /*
@@ -1217,7 +1255,7 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
 
             *output_handle = SQL_NULL_HDESC;
 
-            return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+            return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
         }
 
         if ( SQL_SUCCEEDED( ret ))
@@ -1254,7 +1292,7 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
                            (void *)1);
 #endif
 
-        return function_return( SQL_HANDLE_DBC, connection, ret );
+        return function_return( SQL_HANDLE_DBC, connection, ret, DEFER_R1 );
         }
         break;
 
@@ -1269,7 +1307,7 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
 						ERROR_HY092, NULL,
 						environment -> requested_version );
 
-			return function_return( SQL_HANDLE_ENV, environment, SQL_ERROR );
+			return function_return_nodrv( SQL_HANDLE_ENV, environment, SQL_ERROR );
 		}
 		else if ( __validate_dbc( (DMHDBC) input_handle ))
 		{
@@ -1281,11 +1319,11 @@ SQLRETURN __SQLAllocHandle( SQLSMALLINT handle_type,
 					ERROR_HY092, NULL,
 					connection -> environment -> requested_version );
 	
-			return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+			return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
 		}
 		else
 		{
-			return SQL_ERROR;
+			return SQL_INVALID_HANDLE;
 		}
 		break;
     }
