@@ -130,15 +130,16 @@
 
 static char const rcsid[]= "$RCSfile: SQLBindCol.c,v $ $Revision: 1.8 $";
 
-int check_target_type( int c_type ) 
+int check_target_type( int c_type, int connection_mode) 
 {
     /*
      * driver defined types
      */
-    if ( c_type >= 0x4000 && c_type <= 0x7FFF ) {
+    if ( connection_mode >= SQL_OV_ODBC3_80 && c_type >= 0x4000 && c_type <= 0x7FFF ) {
         return 1;
     }
 
+
 	switch( c_type ) {
 		case SQL_C_CHAR:
 		case SQL_C_LONG:
@@ -180,7 +181,15 @@ int check_target_type( int c_type )
 		case SQL_C_WCHAR:
 		case SQL_ARD_TYPE:
 		case SQL_C_DOUBLE:
-		/* case SQL_C_XML: still trying to find what value this is */
+        /*
+         * MS Added types
+         */
+        case -150:  /* SQL_SS_VARIANT */
+        case -151:  /* SQL_SS_UDT */
+        case -152:  /* SQL_SS_XML */
+        case -153:  /* SQL_SS_TABLE */
+        case -154:  /* SQL_SS_TIME2 */
+        case -155:  /* SQL_SS_TIMESTAMPOFFSET */
 			return 1;
 
 		default:
@@ -253,7 +262,7 @@ SQLRETURN SQLBindCol( SQLHSTMT statement_handle,
                 ERROR_HY090, NULL,
                 statement -> connection -> environment -> requested_version );
 
-        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
+        return function_return_nodrv( SQL_HANDLE_STMT, statement, SQL_ERROR );
     }
 
     /*
@@ -286,7 +295,7 @@ SQLRETURN SQLBindCol( SQLHSTMT statement_handle,
                 ERROR_HY010, NULL,
                 statement -> connection -> environment -> requested_version );
 
-        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
+        return function_return_nodrv( SQL_HANDLE_STMT, statement, SQL_ERROR );
     }
 
 	/*
@@ -294,7 +303,7 @@ SQLRETURN SQLBindCol( SQLHSTMT statement_handle,
      * Its possible to call with the indicator and buffer NULL to unbind without setting the type
 	 */
 
-	if (( target_value || strlen_or_ind ) && !check_target_type( target_type ))
+	if (( target_value || strlen_or_ind ) && !check_target_type( target_type, statement -> connection -> environment -> requested_version ))
 	{
         dm_log_write( __FILE__, 
                 __LINE__, 
@@ -306,7 +315,7 @@ SQLRETURN SQLBindCol( SQLHSTMT statement_handle,
                 ERROR_HY003, NULL,
                 statement -> connection -> environment -> requested_version );
 
-        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
+        return function_return_nodrv( SQL_HANDLE_STMT, statement, SQL_ERROR );
 	}
 
     if ( !CHECK_SQLBINDCOL( statement -> connection ))
@@ -321,7 +330,7 @@ SQLRETURN SQLBindCol( SQLHSTMT statement_handle,
                 ERROR_IM001, NULL,
                 statement -> connection -> environment -> requested_version );
 
-        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
+        return function_return_nodrv( SQL_HANDLE_STMT, statement, SQL_ERROR );
     }
 
     ret = SQLBINDCOL( statement -> connection ,
@@ -347,5 +356,5 @@ SQLRETURN SQLBindCol( SQLHSTMT statement_handle,
                 statement -> msg );
     }
 
-    return function_return( SQL_HANDLE_STMT, statement, ret );
+    return function_return( SQL_HANDLE_STMT, statement, ret, DEFER_R3 );
 }
