@@ -4,7 +4,7 @@
  * (pharvey@codebydesign.com).
  *
  * Modified and extended by Nick Gorham
- * (nick@easysoft.com).
+ * (nick@lurcher.org).
  *
  * Any bugs or problems should be considered the fault of Nick and not
  * Peter.
@@ -27,9 +27,12 @@
  *
  **********************************************************************
  *
- * $Id: SQLSpecialColumns.c,v 1.6 2004/01/12 09:54:39 lurcher Exp $
+ * $Id: SQLSpecialColumns.c,v 1.7 2009/02/18 17:59:08 lurcher Exp $
  *
  * $Log: SQLSpecialColumns.c,v $
+ * Revision 1.7  2009/02/18 17:59:08  lurcher
+ * Shift to using config.h, the compile lines were making it hard to spot warnings
+ *
  * Revision 1.6  2004/01/12 09:54:39  lurcher
  *
  * Fix problem where STATE_S5 stops metadata calls
@@ -125,9 +128,10 @@
  *
  **********************************************************************/
 
+#include <config.h>
 #include "drivermanager.h"
 
-static char const rcsid[]= "$RCSfile: SQLSpecialColumns.c,v $ $Revision: 1.6 $";
+static char const rcsid[]= "$RCSfile: SQLSpecialColumns.c,v $ $Revision: 1.7 $";
 
 SQLRETURN SQLSpecialColumnsA( SQLHSTMT statement_handle,
            SQLUSMALLINT identifier_type,
@@ -187,13 +191,13 @@ SQLRETURN SQLSpecialColumns( SQLHSTMT statement_handle,
     if ( log_info.log_flag )
     {
         sprintf( statement -> msg, "\n\t\tEntry:\
-            \n\t\t\tStatement = %p\
-            \n\t\t\tIdentifier Type = %d\
-            \n\t\t\tCatalog Name = %s\
-            \n\t\t\tSchema Name = %s\
-            \n\t\t\tTable Name = %s\
-            \n\t\t\tScope = %d\
-            \n\t\t\tNullable = %d",
+\n\t\t\tStatement = %p\
+\n\t\t\tIdentifier Type = %d\
+\n\t\t\tCatalog Name = %s\
+\n\t\t\tSchema Name = %s\
+\n\t\t\tTable Name = %s\
+\n\t\t\tScope = %d\
+\n\t\t\tNullable = %d",
                 statement,
                 identifier_type,
                 __string_with_length( s1, catalog_name, name_length1 ), 
@@ -211,40 +215,6 @@ SQLRETURN SQLSpecialColumns( SQLHSTMT statement_handle,
 
     thread_protect( SQL_HANDLE_STMT, statement );
 
-    /*
-     * Check the SQL_ATTR_METADATA_ID settings
-     */
-
-    if ( statement -> metadata_id == SQL_TRUE &&
-            schema_name == NULL )
-    {
-        __post_internal_error( &statement -> error,
-                ERROR_HY009, NULL,
-                statement -> connection -> environment -> requested_version );
-
-        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
-    }
-
-    if ( table_name == NULL )
-    {
-        __post_internal_error( &statement -> error,
-                ERROR_HY009, NULL,
-                statement -> connection -> environment -> requested_version );
-
-        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
-    }
-
-    if (( name_length1 < 0 && name_length1 != SQL_NTS ) ||
-            ( name_length2 < 0 && name_length3 != SQL_NTS ) ||
-            ( name_length3 < 0 && name_length3 != SQL_NTS ))
-    {
-        __post_internal_error( &statement -> error,
-                ERROR_HY090, NULL,
-                statement -> connection -> environment -> requested_version );
-
-        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
-    }
-
     if ( identifier_type != SQL_BEST_ROWID &&
             identifier_type != SQL_ROWVER )
     {
@@ -258,7 +228,49 @@ SQLRETURN SQLSpecialColumns( SQLHSTMT statement_handle,
                 ERROR_HY097, NULL,
                 statement -> connection -> environment -> requested_version );
 
-        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
+        return function_return_nodrv( SQL_HANDLE_STMT, statement, SQL_ERROR );
+    }
+
+    if (( name_length1 < 0 && name_length1 != SQL_NTS ) ||
+         ( name_length2 < 0 && name_length2 != SQL_NTS ))
+    {
+        __post_internal_error( &statement -> error,
+                ERROR_HY090, NULL,
+                statement -> connection -> environment -> requested_version );
+
+        return function_return_nodrv( SQL_HANDLE_STMT, statement, SQL_ERROR );
+    }
+
+    if ( table_name == NULL )
+    {
+        __post_internal_error( &statement -> error,
+                ERROR_HY009, NULL,
+                statement -> connection -> environment -> requested_version );
+
+        return function_return_nodrv( SQL_HANDLE_STMT, statement, SQL_ERROR );
+    }
+
+    if ( name_length3 < 0 && name_length3 != SQL_NTS )
+    {
+        __post_internal_error( &statement -> error,
+                ERROR_HY090, NULL,
+                statement -> connection -> environment -> requested_version );
+
+        return function_return_nodrv( SQL_HANDLE_STMT, statement, SQL_ERROR );
+    }
+
+    /*
+     * Check the SQL_ATTR_METADATA_ID settings
+     */
+
+    if ( statement -> metadata_id == SQL_TRUE &&
+            schema_name == NULL )
+    {
+        __post_internal_error( &statement -> error,
+                ERROR_HY009, NULL,
+                statement -> connection -> environment -> requested_version );
+
+        return function_return_nodrv( SQL_HANDLE_STMT, statement, SQL_ERROR );
     }
 
     if ( scope != SQL_SCOPE_CURROW &&
@@ -275,7 +287,7 @@ SQLRETURN SQLSpecialColumns( SQLHSTMT statement_handle,
                 ERROR_HY098, NULL,
                 statement -> connection -> environment -> requested_version );
 
-        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR ); 
+        return function_return_nodrv( SQL_HANDLE_STMT, statement, SQL_ERROR ); 
     }
 
     if ( nullable != SQL_NO_NULLS &&
@@ -291,7 +303,7 @@ SQLRETURN SQLSpecialColumns( SQLHSTMT statement_handle,
                 ERROR_HY099, NULL,
                 statement -> connection -> environment -> requested_version );
 
-        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
+        return function_return_nodrv( SQL_HANDLE_STMT, statement, SQL_ERROR );
     }
 
     /*
@@ -317,11 +329,14 @@ SQLRETURN SQLSpecialColumns( SQLHSTMT statement_handle,
                 ERROR_24000, NULL,
                 statement -> connection -> environment -> requested_version );
 
-        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR ); 
+        return function_return_nodrv( SQL_HANDLE_STMT, statement, SQL_ERROR ); 
     }
     else if ( statement -> state == STATE_S8 ||
             statement -> state == STATE_S9 ||
-            statement -> state == STATE_S10 )
+            statement -> state == STATE_S10 ||
+            statement -> state == STATE_S13 ||
+            statement -> state == STATE_S14 ||
+            statement -> state == STATE_S15 )
     {
         dm_log_write( __FILE__, 
                 __LINE__, 
@@ -333,7 +348,7 @@ SQLRETURN SQLSpecialColumns( SQLHSTMT statement_handle,
                 ERROR_HY010, NULL,
                 statement -> connection -> environment -> requested_version );
 
-        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
+        return function_return_nodrv( SQL_HANDLE_STMT, statement, SQL_ERROR );
     }
 
     if ( statement -> state == STATE_S11 ||
@@ -351,13 +366,14 @@ SQLRETURN SQLSpecialColumns( SQLHSTMT statement_handle,
                     ERROR_HY010, NULL,
                     statement -> connection -> environment -> requested_version );
 
-            return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
+            return function_return_nodrv( SQL_HANDLE_STMT, statement, SQL_ERROR );
         }
     }
 
     if ( statement -> connection -> unicode_driver )
     {
         SQLWCHAR *s1, *s2, *s3;
+        int wlen;
 
         if ( !CHECK_SQLSPECIALCOLUMNSW( statement -> connection ))
         {
@@ -371,12 +387,15 @@ SQLRETURN SQLSpecialColumns( SQLHSTMT statement_handle,
                     ERROR_IM001, NULL,
                     statement -> connection -> environment -> requested_version );
 
-            return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
+            return function_return_nodrv( SQL_HANDLE_STMT, statement, SQL_ERROR );
         }
 
-        s1 = ansi_to_unicode_alloc( catalog_name, name_length1, statement -> connection );
-        s2 = ansi_to_unicode_alloc( schema_name, name_length2, statement -> connection );
-        s3 = ansi_to_unicode_alloc( table_name, name_length3, statement -> connection );
+        s1 = ansi_to_unicode_alloc( catalog_name, name_length1, statement -> connection, &wlen );
+        name_length1 = wlen;
+        s2 = ansi_to_unicode_alloc( schema_name, name_length2, statement -> connection, &wlen );
+        name_length2 = wlen;
+        s3 = ansi_to_unicode_alloc( table_name, name_length3, statement -> connection, &wlen );
+        name_length3 = wlen;
 
         ret = SQLSPECIALCOLUMNSW( statement -> connection ,
                 statement -> driver_stmt,
@@ -411,7 +430,7 @@ SQLRETURN SQLSpecialColumns( SQLHSTMT statement_handle,
                     ERROR_IM001, NULL,
                     statement -> connection -> environment -> requested_version );
 
-            return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
+            return function_return_nodrv( SQL_HANDLE_STMT, statement, SQL_ERROR );
         }
 
         ret = SQLSPECIALCOLUMNS( statement -> connection ,
@@ -470,5 +489,5 @@ SQLRETURN SQLSpecialColumns( SQLHSTMT statement_handle,
                 statement -> msg );
     }
 
-    return function_return( SQL_HANDLE_STMT, statement, ret );
+    return function_return( SQL_HANDLE_STMT, statement, ret, DEFER_R1 );
 }
