@@ -12,6 +12,7 @@
  **************************************************/
 
 #include <config.h>
+#include <ctype.h>
 #include "isql.h"
 #ifdef HAVE_READLINE
     #include <readline/readline.h>
@@ -31,6 +32,7 @@ static int OpenDatabase( SQLHENV *phEnv, SQLHDBC *phDbc, char *szDSN, char *szUI
 static int ExecuteSQL( SQLHDBC hDbc, char *szSQL, char cDelimiter, int bColumnNames, int bHTMLTable );
 static int ExecuteHelp( SQLHDBC hDbc, char *szSQL, char cDelimiter, int bColumnNames, int bHTMLTable );
 static int ExecuteSlash( SQLHDBC hDbc, char *szSQL, char cDelimiter, int bColumnNames, int bHTMLTable );
+static int ExecuteEcho( SQLHDBC hDbc, char *szSQL, char cDelimiter, int bColumnNames, int bHTMLTable );
 static int	CloseDatabase( SQLHENV hEnv, SQLHDBC hDbc );
 
 static void WriteHeaderHTMLTable( SQLHSTMT hStmt );
@@ -189,6 +191,13 @@ int main( int argc, char *argv[] )
     szSQL = calloc( 1, buffer_size + 1 );
     line_buffer = calloc( 1, buffer_size + 1 );
 
+#ifdef HAVE_SETVBUF
+    /* Ensure result lines are available to reader of whatever stdout */
+    if (bBatch) {
+        (void)setvbuf(stdout, NULL, _IOLBF, (size_t) 0);
+    }
+#endif
+
     /****************************
      * CONNECT
      ***************************/
@@ -214,12 +223,14 @@ int main( int argc, char *argv[] )
             printf( "| \\rollback                             |\n" );
             printf( "| \\tables                               |\n" );
             printf( "| \\columns <table-name>                 |\n" );
+            printf( "| \\echo [string]                        |\n" );
             printf( "| \\quit                                 |\n" );
         }
         else
         {
             printf( "| sql-statement                         |\n" );
             printf( "| help [tablename]                      |\n" );
+            printf( "| echo [string]                         |\n" );
             printf( "| quit                                  |\n" );
         }
         printf( "|                                       |\n" );
@@ -499,6 +510,10 @@ int main( int argc, char *argv[] )
                 {
                     ExecuteHelp( hDbc, szSQL, cDelimiter, bColumnNames, bHTMLTable );
                 }
+                else if ( bufpos >= 4 && memcmp( szSQL, "echo", 4 ) == 0 )
+                {
+                    ExecuteEcho( hDbc, szSQL, cDelimiter, bColumnNames, bHTMLTable );
+                }
                 else
                 {
                     ExecuteSQL( hDbc, szSQL, cDelimiter, bColumnNames, bHTMLTable );
@@ -971,6 +986,19 @@ ExecuteSlash( SQLHDBC hDbc, char *szSQL, char cDelimiter, int bColumnNames, int
             DumpODBCLog( hEnv, hDbc, 0 );
         }
     }
+    else if ( memcmp( szSQL, "echo", 4 ) == 0 )
+    {
+        char *p;
+
+        for ( p = szSQL+4; *p != '\0' && isspace((int)*p) ; ++p )
+            ;
+        if ( *p != '\0' && p == szSQL+4 ) {
+            fprintf( stderr, "[ISQL]ERROR: incorrect echo call\n" );
+            return 0;
+        }
+
+        (void)printf( "%s\n", p );
+    }
     else if ( memcmp( szSQL, "quit", 4 ) == 0 )
     {
         return 0;
@@ -1285,6 +1313,25 @@ ExecuteHelp( SQLHDBC hDbc, char *szSQL, char cDelimiter, int bColumnNames, int b
     return 1;
 }
 
+/****************************
+ * ExecuteEcho - simply write as is the string (if any) to stdout
+ ***************************/
+static int ExecuteEcho( SQLHDBC hDbc, char *szSQL, char cDelimiter, int bColumnNames, int bHTMLTable )
+{
+    char *p;
+
+    for ( p = szSQL+4; *p != '\0' && isspace((int)*p) ; ++p )
+        ;
+    if ( *p != '\0' && p == szSQL+4 ) {
+        fprintf( stderr, "[ISQL]ERROR: incorrect echo call\n" );
+        return 0;
+    }
+
+    (void)printf( "%s\n", p );
+
+    return 1;
+}
+
 
 /****************************
  * CloseDatabase - cleanup in prep for exiting the program
