@@ -11,6 +11,7 @@
  **************************************************/
 
 #include <config.h>
+#include <ctype.h>
 #define UNICODE
 #include "isql.h"
 #include "ini.h"
@@ -33,6 +34,7 @@ static int OpenDatabase( SQLHENV *phEnv, SQLHDBC *phDbc, char *szDSN, char *szUI
 static int CloseDatabase( SQLHENV hEnv, SQLHDBC hDbc );
 static int ExecuteSQL( SQLHDBC hDbc, char *szSQL, char cDelimiter, int bColumnNames, int bHTMLTable );
 static int ExecuteHelp( SQLHDBC hDbc, char *szSQL, char cDelimiter, int bColumnNames, int bHTMLTable );
+static int ExecuteEcho( SQLHDBC hDbc, char *szSQL, char cDelimiter, int bColumnNames, int bHTMLTable );
 
 static void WriteHeaderHTMLTable( SQLHSTMT hStmt );
 static void WriteHeaderDelimited( SQLHSTMT hStmt, char cDelimiter );
@@ -45,6 +47,7 @@ static int DumpODBCLog( SQLHENV hEnv, SQLHDBC hDbc, SQLHSTMT hStmt );
 
 
 int     bVerbose                    = 0;
+int     nUserWidth                  = 0;
 SQLHENV hEnv                        = 0;
 SQLHDBC hDbc                        = 0;
 int     buseED                      = 0;
@@ -126,6 +129,9 @@ int main( int argc, char *argv[] )
                 case 's':
                     buffer_size = atoi( &(argv[nArg][2]) );
                     break;
+                case 'm':
+                    nUserWidth = atoi( &(argv[nArg][2]) );
+                    break;
                 case 'w':
                     bHTMLTable = 1;
                     break;
@@ -164,17 +170,33 @@ int main( int argc, char *argv[] )
             }
             continue;
         }
-        else if ( count == 1 )
-            szDSN = argv[nArg];
-        else if ( count == 2 )
-            szUID = argv[nArg];
-        else if ( count == 3 )
-            szPWD = argv[nArg];
+        else if ( count == 1 ) {
+            if ( nArg < argc ) {
+                szDSN = argv[nArg];
+            }
+        }
+        else if ( count == 2 ) {
+            if ( nArg < argc ) {
+                szUID = argv[nArg];
+            }
+        }
+        else if ( count == 3 ) {
+            if ( nArg < argc ) {
+                szPWD = argv[nArg];
+            }
+        }
         count++;
     }
 
     szSQL = calloc( 1, buffer_size + 1 );
 
+#ifdef HAVE_SETVBUF
+    /* Ensure result lines are available to reader of whatever stdout */
+    if (bBatch) {
+        (void)setvbuf(stdout, NULL, _IOLBF, (size_t) 0);
+    }
+#endif
+
     /****************************
      * CONNECT
      ***************************/
@@ -191,6 +213,7 @@ int main( int argc, char *argv[] )
         printf( "|                                       |\n" );
         printf( "| sql-statement                         |\n" );
         printf( "| help [tablename]                      |\n" );
+        printf( "| echo [string]                         |\n" );
         printf( "| quit                                  |\n" );
         printf( "|                                       |\n" );
         printf( "+---------------------------------------+\n" );
@@ -273,6 +296,8 @@ int main( int argc, char *argv[] )
                 szSQL[1] = '\0';
             else if ( strncmp( szSQL, "help", 4 ) == 0 )
                 ExecuteHelp( hDbc, szSQL, cDelimiter, bColumnNames, bHTMLTable );
+            else if ( strncmp( szSQL, "echo", 4 ) == 0 )
+                ExecuteEcho( hDbc, szSQL, cDelimiter, bColumnNames, bHTMLTable );
             else if (memcmp(szSQL, "--", 2) != 0)
                 ExecuteSQL( hDbc, szSQL, cDelimiter, bColumnNames, bHTMLTable );
         }
@@ -287,6 +312,42 @@ int main( int argc, char *argv[] )
     exit( 0 );
 }
 
+/****************************
+ * OptimalDisplayWidth
+ ***************************/
+static SQLUINTEGER
+OptimalDisplayWidth( SQLHSTMT hStmt, SQLINTEGER nCol, int nUserWidth )
+{
+    SQLUINTEGER nLabelWidth                     = 10;
+    SQLULEN nDataWidth                      = 10;
+    SQLUINTEGER nOptimalDisplayWidth            = 10;
+    SQLCHAR     szColumnName[MAX_DATA_WIDTH+1]; 
+
+    *szColumnName = '\0';
+
+    SQLColAttribute( hStmt, nCol, SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, (SQLLEN*)&nDataWidth );
+    SQLColAttribute( hStmt, nCol, SQL_DESC_LABEL, szColumnName, sizeof(szColumnName), NULL, NULL );
+    nLabelWidth = strlen((char*) szColumnName );
+
+    /*
+     * catch sqlserver var(max) types
+     */
+
+    if ( nDataWidth == 0 ) {
+        nDataWidth = MAX_DATA_WIDTH;
+    }
+
+    nOptimalDisplayWidth = max( nLabelWidth, nDataWidth );
+
+    if ( nUserWidth > 0 )
+        nOptimalDisplayWidth = min( nOptimalDisplayWidth, nUserWidth );
+
+    if ( nOptimalDisplayWidth > MAX_DATA_WIDTH )
+        nOptimalDisplayWidth = MAX_DATA_WIDTH;
+
+    return nOptimalDisplayWidth;
+}
+
 /****************************
  * OpenDatabase - do everything we have to do to get a viable connection to szDSN
  ***************************/
@@ -602,6 +663,24 @@ static int ExecuteHelp( SQLHDBC hDbc, char *szSQL, char cDelimiter, int bColumnN
     return 1;
 }
 
+/****************************
+ * ExecuteEcho - simply write as is the string (if any) to stdout
+ ***************************/
+static int ExecuteEcho( SQLHDBC hDbc, char *szSQL, char cDelimiter, int bColumnNames, int bHTMLTable )
+{
+    char *p;
+
+    for ( p = szSQL+4; *p != '\0' && isspace((int)*p) ; ++p )
+        ;
+    if ( *p != '\0' && p == szSQL+4 ) {
+        fprintf( stderr, "[ISQL]ERROR: incorrect echo call\n" );
+        return 0;
+    }
+
+    (void)printf( "%s\n", p );
+
+    return 1;
+}
 
 /****************************
  * CloseDatabase - cleanup in prep for exiting the program
@@ -780,10 +859,10 @@ void UWriteHeaderNormal( SQLHSTMT hStmt, SQLTCHAR *szSepLine )
 {
     SQLINTEGER      nCol                            = 0;
     SQLSMALLINT     nColumns                        = 0;
-    SQLULEN         nMaxLength                      = 10;
     SQLTCHAR            szColumn[MAX_DATA_WIDTH+20];    
     SQLTCHAR            szColumnName[MAX_DATA_WIDTH+1]; 
     SQLTCHAR            szHdrLine[32001];   
+    SQLUINTEGER     nOptimalDisplayWidth            = 10;
 
     szColumn[ 0 ]       = 0;    
     szColumnName[ 0 ]   = 0;    
@@ -794,20 +873,20 @@ void UWriteHeaderNormal( SQLHSTMT hStmt, SQLTCHAR *szSepLine )
 
     for ( nCol = 1; nCol <= nColumns; nCol++ )
     {
-        SQLColAttribute( hStmt, nCol, SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, (SQLLEN*)&nMaxLength );
+        nOptimalDisplayWidth = OptimalDisplayWidth( hStmt, nCol, nUserWidth );
         SQLColAttribute( hStmt, nCol, SQL_DESC_LABEL, szColumnName, sizeof(szColumnName), NULL, NULL );
-        if ( nMaxLength > MAX_DATA_WIDTH ) nMaxLength = MAX_DATA_WIDTH;
+        if ( nOptimalDisplayWidth > MAX_DATA_WIDTH ) nOptimalDisplayWidth = MAX_DATA_WIDTH;
 
         uc_to_ascii( szColumnName );
 
         /* SEP */
         memset( szColumn, '\0', sizeof(szColumn) );
-        memset( szColumn, '-', max( nMaxLength, strlen((char*)szColumnName) ) + 1 );
+        memset( szColumn, '-', max( nOptimalDisplayWidth, strlen((char*)szColumnName) ) + 1 );
         strcat((char*) szSepLine, "+" );
         strcat((char*) szSepLine,(char*) szColumn );
 
         /* HDR */
-        sprintf((char*) szColumn, "| %-*s", (int)max( nMaxLength, strlen((char*)szColumnName) ), (char*)szColumnName );
+        sprintf((char*) szColumn, "| %-*s", (int)max( nOptimalDisplayWidth, strlen((char*)szColumnName) ), (char*)szColumnName );
         strcat((char*) szHdrLine,(char*) szColumn );
     }
     strcat((char*) szSepLine, "+\n" );
@@ -826,10 +905,10 @@ static SQLLEN WriteBodyNormal( SQLHSTMT hStmt )
     SQLTCHAR        szColumn[MAX_DATA_WIDTH+20];
     SQLTCHAR        szColumnValue[MAX_DATA_WIDTH+1];
     SQLTCHAR        szColumnName[MAX_DATA_WIDTH+1]; 
-    SQLULEN         nMaxLength                      = 10;
     SQLRETURN       nReturn                         = 0;
     SQLRETURN       ret;
     SQLLEN          nRows                           = 0;
+    SQLUINTEGER     nOptimalDisplayWidth            = 10;
 
     szColumn[ 0 ]       = 0;
     szColumnValue[ 0 ]  = 0;
@@ -845,21 +924,21 @@ static SQLLEN WriteBodyNormal( SQLHSTMT hStmt )
         for ( nCol = 1; nCol <= nColumns; nCol++ )
         {
             SQLColAttribute( hStmt, nCol, SQL_DESC_LABEL, szColumnName, sizeof(szColumnName), NULL, NULL );
-            SQLColAttribute( hStmt, nCol, SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, (SQLLEN*)&nMaxLength );
+            nOptimalDisplayWidth = OptimalDisplayWidth( hStmt, nCol, nUserWidth );
 
             uc_to_ascii( szColumnName );
 
-            if ( nMaxLength > MAX_DATA_WIDTH ) nMaxLength = MAX_DATA_WIDTH;
+            if ( nOptimalDisplayWidth > MAX_DATA_WIDTH ) nOptimalDisplayWidth = MAX_DATA_WIDTH;
             nReturn = SQLGetData( hStmt, nCol, SQL_C_WCHAR, (SQLPOINTER)szColumnValue, sizeof(szColumnValue), &nIndicator );
             szColumnValue[MAX_DATA_WIDTH] = '\0';
             uc_to_ascii( szColumnValue );
 
             if ( nReturn == SQL_SUCCESS && nIndicator != SQL_NULL_DATA )
             {
-                if ( strlen((char*)szColumnValue) < max( nMaxLength, strlen((char*)szColumnName )))
+                if ( strlen((char*)szColumnValue) < max( nOptimalDisplayWidth, strlen((char*)szColumnName )))
                 {
                     int i;
-                    size_t maxlen=max( nMaxLength, strlen((char*)szColumnName ));
+                    size_t maxlen=max( nOptimalDisplayWidth, strlen((char*)szColumnName ));
                     strcpy((char*) szColumn, "| " );
                     strcat((char*) szColumn, (char*) szColumnValue );
 
@@ -881,7 +960,7 @@ static SQLLEN WriteBodyNormal( SQLHSTMT hStmt )
             }
             else
             {
-                sprintf((char*)  szColumn, "| %-*s", (int)max( nMaxLength, strlen((char*) szColumnName) ), "" );
+                sprintf((char*)  szColumn, "| %-*s", (int)max( nOptimalDisplayWidth, strlen((char*) szColumnName) ), "" );
             }
             fputs((char*)  szColumn, stdout );
         }
